import configparser
import logging
import os
from dataclasses import dataclass
from typing import Iterable
from version import VERSION_ACTIONLINT_TXT, reset_dev_version, get_actionlint_version, get_pip_version, VERSION

import requests
import semver
from requests_html import HTMLSession

README_MD = os.path.join(os.path.dirname(__file__), "..", "README.md")
assert os.path.isfile(README_MD), (os.getcwd(), README_MD)

ACTIONLINT_RELEASES = "https://github.com/rhysd/actionlint/releases/"  # must end with "/"
SETUP_CFG = os.path.join(os.path.dirname(__file__), "checksums.cfg")
GITHUB_OUT = os.getenv("GITHUB_OUTPUT")

logging.basicConfig(level=logging.INFO)
log = logging.getLogger()


def get_release_page_links():
    session = HTMLSession()
    r = session.get(ACTIONLINT_RELEASES)
    return r.html.links


def get_checksum_file(newest_release_link: str) -> str:
    checksums_file = requests.get("https://github.com" + newest_release_link)
    # checksum_file_name = newest_release_link.split('/')[-1]
    content = checksums_file.content.decode("utf-8")
    return content


def get_newest_release_link(links: Iterable[str]) -> str:
    newest_release_link = None
    for link in links:
        link: str
        if link.endswith("_checksums.txt"):
            newest_release_link = link
            break
    return newest_release_link


def update_config(checksum_file_content: str, current_version: str, newest_version_str: str) -> None:
    """Update tag and checksum in config file

    Config file format:

    ```
    [python_platform-python_machine]
    file_name=actionlint_1.6.25_linux_amd64.tar.gz  ; format of this file is important
    checksum=...
    url=...  ; download url
    ```
    """
    config = configparser.ConfigParser()
    config.read(SETUP_CFG)
    checksums: dict[str, ChecksumInfo] = {c.filename: c for c in get_checksums(checksum_file_content)}
    for sec in config.sections():
        platform_config = config[sec]
        new_file_name = platform_config["file_name"].replace(current_version, newest_version_str)
        platform_config["file_name"] = new_file_name
        url = f"https://github.com/rhysd/actionlint/releases/download/v{newest_version_str}/{new_file_name}"
        platform_config["url"] = url
        platform_config["checksum"] = checksums[new_file_name].checksum

    with open(SETUP_CFG, "w") as file:
        config.write(file, space_around_delimiters=True)


def update_actionlint_version(newest_version_str):
    with open(VERSION_ACTIONLINT_TXT, "w") as file:
        file.write(newest_version_str)
    reset_dev_version()  # just to be sure


def update_readme(current_version: str, new_version: str):
    """execute as last - the order of version is re-read from files"""
    with open(README_MD, "r") as readme_ro:
        contents = readme_ro.read()
        contents = contents.replace(VERSION, get_pip_version())
        contents = contents.replace(current_version, new_version)
    with open(README_MD, "w") as readme_rw:
        readme_rw.write(contents)


@dataclass(frozen=True)
class ChecksumInfo:
    version: str
    filename: str
    postfix: str
    checksum: str


def get_checksums(checksum_file_content):
    for line in checksum_file_content.split("\n"):
        if not line:
            continue
        checksum, file = line.split()
        exe_name, version, platform, machine_with_extension = file.split("_")
        file_ending = "_" + platform + "_" + machine_with_extension
        yield ChecksumInfo(
            version=checksum,
            checksum=checksum,
            filename=file,
            postfix=file_ending,
        )


def write_github_output(newest_version, newest_version_str, is_update_required: bool):
    if GITHUB_OUT is None:
        return
    with open(GITHUB_OUT, "a") as file:
        file.write(f"version={newest_version_str}\n")
        file.write(f"update_required={is_update_required}\n")
        file.write(f"release_url={ACTIONLINT_RELEASES}tag/v{newest_version}\n")


def main():
    links = get_release_page_links()
    newest_release_link = get_newest_release_link(links)
    newest_version_str = newest_release_link.split("/")[-2].lstrip("v")
    log.info(f"Newest version: {newest_version_str}")
    newest_version = semver.Version.parse(newest_version_str)
    current_version = semver.Version.parse(get_actionlint_version())
    if newest_version.compare(current_version) != 1:
        log.info("Local version is newest, all good. Exiting.")
        write_github_output(newest_version, newest_version_str, is_update_required=False)
        exit(0)
    checksum_file_content = get_checksum_file(newest_release_link)
    update_config(checksum_file_content, str(current_version), newest_version_str)
    update_actionlint_version(newest_version_str)
    update_readme(str(current_version), newest_version_str)
    log.info("Local file 'checksums.cfg' and 'VERSION_ACTIONLINT.txt' and 'README.md' updated successfully. ")
    log.warning("A new commit is required.")
    write_github_output(newest_version, newest_version_str, is_update_required=True)


if __name__ == "__main__":
    main()



from wheel.bdist_wheel import bdist_wheel as orig_bdist_wheel


class bdist_wheel(orig_bdist_wheel):
    def finalize_options(self):
        orig_bdist_wheel.finalize_options(self)
        # Mark us as not a pure python package
        self.root_is_pure = False

    def get_tag(self):
        _, _, plat = orig_bdist_wheel.get_tag(self)
        # We don't contain any python source, nor any python extensions
        return "py2.py3", "none", plat



import os
from argparse import ArgumentParser

_this_dir = os.path.dirname(__file__)
# VERSION_ACTIONLINT.txt is version of actionlint library
VERSION_ACTIONLINT_TXT = os.path.join(_this_dir, "VERSION_ACTIONLINT.txt")
VERSION_BUILD_SYSTEM_TXT = os.path.join(_this_dir, "VERSION_BUILD_SYSTEM.txt")
VERSION_DEV_TXT = os.path.join(_this_dir, "VERSION_DEV.txt")
GITHUB_OUT = os.getenv("GITHUB_OUTPUT")


def get_pip_version():
    global VERSION
    with open(VERSION_ACTIONLINT_TXT) as f:
        # VERSION_BUILD_SYSTEM.txt is version of this build system
        with open(VERSION_BUILD_SYSTEM_TXT) as f_build:
            with open(VERSION_DEV_TXT) as f_dev:
                # used by pyproject.toml
                v = ".".join([f.read().strip(), f_build.read().strip()])
                __dev_version = f_dev.read().strip()
                if __dev_version != "0":
                    v += f".dev.{__dev_version}"
                return v


def get_actionlint_version():
    with open(VERSION_ACTIONLINT_TXT) as r:
        return r.read().strip()


VERSION = get_pip_version()


def increment_build_version():
    with open(VERSION_BUILD_SYSTEM_TXT) as f:
        version = int(f.read().strip())
    with open(VERSION_BUILD_SYSTEM_TXT, "w") as f_w:
        f_w.write(f"{version + 1}\n")


def increment_dev_version():
    with open(VERSION_DEV_TXT) as f:
        version = int(f.read().strip())
    with open(VERSION_DEV_TXT, "w") as f_w:
        f_w.write(f"{version + 1}\n")


def reset_dev_version():
    with open(VERSION_DEV_TXT, "w") as f_w:
        f_w.write(f"0\n")


def main():
    args = ArgumentParser()
    args.add_argument("--release", help="error if in version contains '.dev.N' string", action="store_true")
    args.add_argument("--increment-build", help="increment VERSION_BUILD_SYSTEM.txt", action="store_true")
    args.add_argument("--increment-dev", help="increment VERSION_DEV.txt", action="store_true")
    args.add_argument("--reset-dev", help="set VERSION_DEV.txt to 0", action="store_true")
    return args.parse_args()


README = os.path.join(_this_dir, "..", "README.md")


def fix_readme(old_version: str, new_version: str):
    with open(README, "r") as f_r:
        f_content = f_r.read()
    with open(README, "w") as f_w:
        new_f_content = f_content.replace(old_version, new_version)
        f_w.write(new_f_content)


if __name__ == "__main__":
    args = main()
    if args.release and ".dev." in VERSION:
        print(f"ERROR: the version is {VERSION} and should not contain .dev.N")
        exit(1)
    prev_version = get_pip_version()
    if args.increment_build:
        increment_build_version()
    if args.increment_dev:
        increment_dev_version()
    if args.reset_dev:
        reset_dev_version()
    current_version = get_pip_version()
    if prev_version != current_version:
        fix_readme(prev_version, current_version)
    print(current_version)



import configparser
import os
import platform
import sys

from setuptools import Command

from ..utils.file_ops import download
from ..utils.file_ops import extract
from ..utils.file_ops import save_executable

SETUP_CFG = os.path.join(os.path.dirname(__file__), "..", "checksums.cfg")


def get_architecture():
    if platform.machine().lower() == "aarch64":
        return "arm64"

    return platform.machine()


class fetch_binaries(Command):
    description = "fetch binaries based on config in checksums.cfg"
    build_temp = None
    user_options: "list[tuple]" = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        self.set_undefined_options("build", ("build_temp", "build_temp"))

    def run(self):
        # save binary to self.build_temp
        config = configparser.ConfigParser()
        config.read(SETUP_CFG)
        section = sys.platform + "-" + get_architecture()
        url = config[section]["url"]
        sha256 = config[section]["checksum"]
        archive = download(url, sha256)
        data = extract(url, archive)
        out = save_executable(data, self.build_temp)
        print(f"Downloaded executable for {section}: {out}")



from distutils.command.build import build as orig_build


# todo do not change to setuptools it causes infinite recursion. Do not know why...
# from setuptools.command.build_ext import build_ext as orig_build


class build(orig_build):
    sub_commands = orig_build.sub_commands + [("fetch_binaries", None)]



from setuptools import Command


class install_actionlint(Command):
    description = "install the actionlint executable"
    outfiles = ()
    build_dir = install_dir = None
    user_options: "list[tuple]" = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        # this initializes attributes based on other commands' attributes
        self.set_undefined_options("build", ("build_temp", "build_dir"))
        self.set_undefined_options(
            "install",
            ("install_scripts", "install_dir"),
        )

    def run(self):
        self.outfiles = self.copy_tree(self.build_dir, self.install_dir)

    def get_outputs(self):
        return self.outfiles
